'use strict';

/**
 * Module dependencies.
 */

var facade = require('segmentio-facade');
var each = require('component-each');
var integration = require('@segment/analytics.js-integration');
var useHttps = require('use-https');
var Identify = facade.Identify;
var Track = facade.Track;

/**
 * Expose `TellApart`.
 */

var TellApart = module.exports = integration('TellApart')
  .global('TellApartCrumb')
  .option('siteId', '')
  .tag('http', '<script src="http://static.tellaparts.com/{{ siteId }}/crumb.js">')
  .tag('https', '<script src="https://sslt.tellapart.com/{{ siteId }}/crumb.js">');

/**
 * Initialize.
 */

TellApart.prototype.initialize = function() {
  window.__cmbLoaded = false;
  var name = useHttps() ? 'https' : 'http';
  var ready = this.ready;
  this.load(name, function() {
    window.__cmbLoaded = true;
    ready();
  });
};

/**
 * Track Product List Viewed.
 *
 * This just calls the abstracted TrackPage with type 'ProductCategory'.
 *
 * @param {Track} track
 */

TellApart.prototype.productListViewed = function(track) {
  this.trackPage('ProductCategory', track);
};

/**
 * Track product viewed.
 *
 * This just calls the abstracted TrackPage with type 'Product'.
 *
 * @param {Track} track
 */

TellApart.prototype.productViewed = function(track) {
  this.trackPage('Product', track);
};

/**
 * Track pages to TellApart for viewing products and categories.
 *
 * @param {String} pageType
 * @param {Track} track
 */

TellApart.prototype.trackPage = function(pageType, track) {
  var siteId = this.options.siteId;
  var user = this.analytics.user();
  var userId = user.id();
  var category = track.category();
  var sku = track.sku();

  var action = window.TellApartCrumb.makeCrumbAction(siteId, 'pv');
  action.setActionAttr('PageType', pageType);
  if (category) action.setActionAttr('ProductCategoryPath', category);
  if (sku) action.setActionAttr('SKU', sku);
  action.setMerchantUserId(userId || '');
  action.finalize();
};

/**
 * Track added product to the cart.
 *
 * Sends along a PartialAdd to trackCartUpdate.
 *
 * @param {Track} track
 */

TellApart.prototype.productAdded = function(track) {
  this.trackCartUpdate('PartialAdd', track);
};

/**
 * Track product removed from the cart.
 *
 * Sends along a PartialRemove to trackCartUpdate.
 *
 * @param {Track} track
 */

TellApart.prototype.productRemoved = function(track) {
  this.trackCartUpdate('PartialRemove', track);
};

/**
 * Track cart updates.
 *
 * The `UpdateCartType` attribute can be:
 * - Full (the default)
 * - PartialAdd
 * - PartialRemove
 *
 * Completed Order update cart events will include the full cart contents, and the
 * `Partial*` methods will be used in the shopping period for individual adds/removes.
 *
 * @param {String} updateType
 * @param {Track} track
 */

TellApart.prototype.trackCartUpdate = function(updateType, track) {
  var siteId = this.options.siteId;
  var user = this.analytics.user();
  var userId = user.id();
  var products = track.products();
  if (products.length === 0) products = [track.properties()];

  var action = window.TellApartCrumb.makeCrumbAction(siteId, 'updatecart');
  action.setActionAttr('UpdateCartType', updateType);
  addProducts(action, products);
  action.setMerchantUserId(userId || '');
  action.finalize();
  return action;
};

/**
 * Track order completed.
 *
 * @param {Track} track
 */

TellApart.prototype.orderCompleted = function(track) {
  var user = this.analytics.user();
  var siteId = this.options.siteId;
  var currency = track.currency();
  var orderId = track.orderId();
  var total = track.total();
  var userId = user.id();
  var e = email(user);

  var action = window.TellApartCrumb.makeCrumbAction(siteId, 'tx');
  if (e) action.setActionAttr('Email', e);
  if (orderId) action.setActionAttr('TransactionId', orderId);
  if (total) action.setActionAttr('TransactionTotal', total);
  action.setActionAttr('TransactionTotalCurrency', currency);
  addProducts(action, track.products());
  action.setMerchantUserId(userId || '');
  action.finalize();
  return action;
};

/**
 * Add products to action.
 *
 * @param {Object} action Result from `TellApartCrumb.makeCrumbAction`
 * @param {Array} products
 */

function addProducts(action, products) {
  each(products, function(product) {
    var track = new Track({ properties: product });
    var sku = track.sku();
    var price = track.price();

    action.beginItem();
    if (sku) action.setItemAttr('SKU', sku);
    if (price) action.setItemAttr('ProductPrice', price);
    action.setItemAttr('ProductCurrency', track.currency());
    action.setItemAttr('ItemCount', track.quantity());
    action.endItem();
  });
}

/**
 * Get email from user.
 *
 * @param {Object} user
 * @return {String}
 */

function email(user) {
  var identify = new Identify({ userId: user.id(), traits: user.traits() });
  return identify.email();
}
