'use strict';

var Analytics = require('@segment/analytics.js-core').constructor;
var integration = require('@segment/analytics.js-integration');
var sandbox = require('@segment/clear-env');
var tester = require('@segment/analytics.js-integration-tester');
var TellApart = require('../lib/');

describe('TellApart', function() {
  var analytics;
  var spy;
  var tellApart;
  var options = {
    siteId: 'Wpwr5DMdEcM3'
  };

  beforeEach(function() {
    analytics = new Analytics();
    tellApart = new TellApart(options);
    analytics.use(TellApart);
    analytics.use(tester);
    analytics.add(tellApart);
  });

  afterEach(function() {
    analytics.restore();
    analytics.reset();
    tellApart.reset();
    sandbox();
  });

  it('should have the right settings', function() {
    analytics.compare(TellApart, integration('TellApart')
      .option('siteId', ''));
  });

  describe('after loading', function() {
    beforeEach(function(done) {
      analytics.once('ready', done);
      analytics.initialize();
      analytics.page();
    });

    describe('#track', function() {
      stubAction();

      it('should track productListViewed, passing in sku and category', function() {
        analytics.identify('id');
        analytics.track('product list viewed', { category: 'bracelets > chains' });
        var action = spy.returns[0];
        analytics.assert(action);
        analytics.called(action.setActionAttr, 'PageType', 'ProductCategory');
        analytics.called(action.setActionAttr, 'ProductCategoryPath', 'bracelets > chains');
        analytics.called(action.setMerchantUserId, 'id');
        analytics.called(action.finalize);
      });

      it('should track productViewed, passing in sku and category', function() {
        analytics.identify('id');
        analytics.track('product viewed', {
          sku: '123',
          category: 'bracelets'
        });
        var action = spy.returns[0];
        analytics.assert(action);
        analytics.called(action.setActionAttr, 'PageType', 'Product');
        analytics.called(action.setActionAttr, 'ProductCategoryPath', 'bracelets');
        analytics.called(action.setActionAttr, 'SKU', '123');
        analytics.called(action.setMerchantUserId, 'id');
        analytics.called(action.finalize);
      });

      it('should track productAdded with the added product', function() {
        analytics.identify('id');
        analytics.track('product added', {
          sku: 'xyz',
          price: 29.99,
          quantity: 1,
          currency: 'EUR'
        });
        var action = spy.returns[0];
        analytics.assert(action);
        analytics.called(action.setActionAttr, 'UpdateCartType', 'PartialAdd');
        analytics.called(action.setMerchantUserId, 'id');
        analytics.calledOnce(action.beginItem);
        analytics.calledOnce(action.endItem);
        analytics.called(action.setItemAttr, 'SKU', 'xyz');
        analytics.called(action.setItemAttr, 'ProductPrice', 29.99);
        analytics.called(action.setItemAttr, 'ProductCurrency', 'EUR');
        analytics.called(action.setItemAttr, 'ItemCount', 1);
        analytics.called(action.finalize);
      });

      it('should track productRemoved with the removed product', function() {
        analytics.identify('id');
        analytics.track('product removed', {
          sku: 'abc',
          price: 10,
          quantity: 3
        });
        var action = spy.returns[0];
        analytics.assert(action);
        analytics.called(action.setActionAttr, 'UpdateCartType', 'PartialRemove');
        analytics.called(action.setMerchantUserId, 'id');
        analytics.calledOnce(action.beginItem);
        analytics.calledOnce(action.endItem);
        analytics.called(action.setItemAttr, 'SKU', 'abc');
        analytics.called(action.setItemAttr, 'ProductPrice', 10);
        analytics.called(action.setItemAttr, 'ProductCurrency', 'USD');
        analytics.called(action.setItemAttr, 'ItemCount', 3);
        analytics.called(action.finalize);
      });

      it('should track orderCompleted', function() {
        var products = [
          { sku: 'abc', price: 10, quantity: 3 },
          { sku: 'xyz', price: 29.99, quantity: 1 }
        ];
        analytics.identify('id', { email: 'email@email.com' });
        analytics.track('order completed', {
          orderId: 'order-id',
          total: 59.99,
          products: products
        });
        var action = spy.returns[0];
        analytics.assert(action);
        analytics.called(action.setActionAttr, 'Email', 'email@email.com');
        analytics.called(action.setActionAttr, 'TransactionId', 'order-id');
        analytics.called(action.setActionAttr, 'TransactionTotal', 59.99);
        analytics.called(action.setActionAttr, 'TransactionTotalCurrency', 'USD');
        analytics.called(action.setMerchantUserId, 'id');
        analytics.calledTwice(action.beginItem);
        analytics.calledTwice(action.endItem);
        analytics.called(action.setItemAttr, 'SKU', 'abc');
        analytics.called(action.setItemAttr, 'ProductPrice', 10);
        analytics.called(action.setItemAttr, 'ProductCurrency', 'USD');
        analytics.called(action.setItemAttr, 'ItemCount', 3);
        analytics.called(action.setItemAttr, 'SKU', 'xyz');
        analytics.called(action.setItemAttr, 'ProductPrice', 29.99);
        analytics.called(action.setItemAttr, 'ProductCurrency', 'USD');
        analytics.called(action.setItemAttr, 'ItemCount', 1);
        analytics.called(action.finalize);
      });
    });
  });

  function stubAction() {
    beforeEach(function() {
      analytics.stub(window.TellApartCrumb, 'makeCrumbAction');
      spy = window.TellApartCrumb.makeCrumbAction;
      window.TellApartCrumb.makeCrumbAction = function(id, type) {
        spy.apply(this, arguments);
        var action = new Action(id, type);
        spy.returns[spy.returns.length - 1] = action;
        return action;
      };
    });
  }

  /**
   * Stub tellApart action object.
   */

  function Action(id, type) {
    if (!(this instanceof Action)) return new Action(id, type);

    analytics.stub(this, 'setActionAttr');
    analytics.stub(this, 'setMerchantUserId');
    analytics.stub(this, 'setItemAttr');
    analytics.stub(this, 'beginItem');
    analytics.stub(this, 'endItem');
    analytics.stub(this, 'finalize');
  }
});
